from __future__ import annotations
from abc import abstractmethod

from omnistore.store import Store


class ObjStore(Store):
    @abstractmethod
    def __init__(self, endpoint: str, bucket: str):
        """
        Construct a new client to communicate with the provider.
        """
        pass



from operator import truediv
from pathlib import Path

import oss2
from oss2.credentials import EnvironmentVariableCredentialsProvider

from omnistore.objstore.objstore import ObjStore


class OSS(ObjStore):
    def __init__(self, endpoint: str, bucket: str):
        # Make sure environments OSS_ACCESS_KEY_ID and OSS_ACCESS_KEY_SECRET are exist.
        auth = oss2.ProviderAuth(EnvironmentVariableCredentialsProvider())
        self._bucket = oss2.Bucket(auth, endpoint, bucket)

    def create_dir(self, dirname: str):
        self._bucket.put_object(dirname, "")

    def delete_dir(self, dirname: str):
        for obj in oss2.ObjectIterator(self._bucket, prefix=dirname):
            self._bucket.delete_object(obj.key)

    def upload(self, src: str, dest: str):
        oss2.resumable_upload(self._bucket, dest, src)

    def upload_dir(self, src_dir: str, dest_dir: str):
        for file in Path(src_dir).rglob("*"):
            if file.is_file():
                self.upload(file, dest_dir + "/" + file.name)
                continue

            if file.is_dir():
                # TODO: Support uploading subdirectory.
                print("Don't support uploading subdirectory yet")

    def download(self, src: str, dest: str):
        oss2.resumable_download(self._bucket, src, dest)

    def download_dir(self, src_dir: str, dest_dir: str):
        if not src_dir.endswith("/"):
            src_dir += "/"

        if not dest_dir.endswith("/"):
            dest_dir += "/"

        path = Path(dest_dir)
        if not path.exists():
            path.mkdir(parents=True, exist_ok=True)

        for obj in oss2.ObjectIterator(self._bucket, prefix=src_dir, delimiter="/"):
            if obj.is_prefix():  # If this is a folder
                # TODO: This is enough for download models, but not enough for general download usage.
                print(f"Don't support downloading subdirectory: {obj.key} yet")
            else:  # If this is a file
                self.download(obj.key, dest_dir + obj.key.split("/")[-1])

    def delete(self, filename: str):
        return self._bucket.delete_object(filename)

    def exists(self, filename: str):
        return self._bucket.object_exists(filename)



from abc import ABC, abstractmethod


class Store(ABC):
    @abstractmethod
    def create_dir(self, dirname: str):
        """
        create_dir will create a dir in the provider.
        """
        pass

    @abstractmethod
    def delete_dir(self, dirname: str):
        """
        delete_dir will delete a dir from the provider, together with all the files under the directory.
        """
        pass

    @abstractmethod
    def upload(self, src: str, dest: str):
        """
        upload will upload the file from src to dest, both parameters are file names.
        """
        pass

    @abstractmethod
    def upload_dir(self, src_dir: str, dest_dir: str):
        """
        upload_dir will upload the folder from src_dir to dest_dir, both parameters are folder names.
        """
        pass

    @abstractmethod
    def download(self, src: str, dest: str):
        """
        download will download the file from src to dest, both parameters are file names.
        """
        pass

    @abstractmethod
    def download_dir(self, src_dir: str, dest_dir: str):
        """
        download_dir will download the folder from src_dir to dest_dir, both parameters are folder names.
        """
        pass

    @abstractmethod
    def delete(self, filename: str):
        """
        delete will delete the file from the provider.
        """
        pass

    @abstractmethod
    def exists(self, filename: str):
        """
        exists checks whether the file exists in the provider.
        """
        pass



from omnistore.objstore.aliyun_oss import OSS
from omnistore.store import Store

OBJECT_STORE_OSS = "OSS"


class StoreFactory:
    ObjStores = {
        OBJECT_STORE_OSS: OSS,
    }

    @classmethod
    def new_client(cls, provider: str, endpoint: str, bucket: str) -> Store:
        objstore = cls.ObjStores[provider]
        if not objstore:
            raise KeyError(f"Unknown object store provider {provider}")

        return objstore(endpoint=endpoint, bucket=bucket)
